@@ -1,4 +1,4 @@
-;;; hui-jmenu.el ---  Popup menus for jumping to and managing buffers, frames, and windows
+;;; hui-jmenu.el --- Popup menus for jumping to and managing buffers, frames, and windows
 ;;
 ;; Author:       Bob Weiner
 ;;
@@ -141,13 +141,12 @@
 ;;; ************************************************************************
 
 (defun hui-menu-buffer-mode-name (buffer)
-  (let ((mname (cdr (assq 'mode-name (buffer-local-variables buffer)))))
+  (let ((mname (buffer-local-value 'mode-name buffer)))
     (if mname
 	;; Next line needed to ensure mode name is always formatted as
 	;; a string.
 	(format-mode-line mname)
-      (capitalize (symbol-name
-		   (cdr (assq 'major-mode (buffer-local-variables buffer))))))))
+      (capitalize (symbol-name (buffer-local-value 'major-mode buffer))))))
 
 (defun hui-menu-frame-name (frame)
   "Return the name of FRAME."
