@@ -4,7 +4,7 @@
 ;;
 ;; Orig-Date:    04-Feb-89
 ;;
-;; Copyright (C) 1991-2017  Free Software Foundation, Inc.
+;; Copyright (C) 1991-2018  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -95,6 +95,12 @@ Its default value is #'smart-scroll-down."
 
 (defvar hkey-alist
   '(
+    ;; Company completion mode
+    ((and (boundp 'company-active-map)
+	  (memq company-active-map (current-minor-mode-maps))) .
+	  ((smart-company-to-definition) . (smart-company-help)))
+    ;;
+    ;; Treemacs hierarchical file manager
     ((eq major-mode 'treemacs-mode) . 
      ((smart-treemacs) . (smart-treemacs)))
     ;;
@@ -235,6 +241,14 @@ Its default value is #'smart-scroll-down."
     ((eq major-mode 'pages-directory-mode) .
      ((pages-directory-goto) . (pages-directory-goto)))
     ;;
+    ;; Python files - ensure this comes before Imenu for more advanced
+    ;; definition lookups
+    ((and (or (and (eq major-mode 'python-mode) buffer-file-name)
+	      (let ((case-fold-search))
+		(string-match "\\`\\(Pydoc:\\|\\*?Python\\)" (buffer-name))))
+	  (setq hkey-value (smart-python-at-tag-p))) .
+	  ((smart-python hkey-value) . (smart-python hkey-value 'next-tag)))
+    ;;
     ;; Imenu listing in GNU Emacs
     ((smart-imenu-item-at-p)
      . ((smart-imenu-display-item-where (car hkey-value) (cdr hkey-value)) .
@@ -258,8 +272,11 @@ Its default value is #'smart-scroll-down."
 	  buffer-file-name (smart-asm-at-tag-p)) .
 	  ((smart-asm) . (smart-asm nil 'next-tag)))
     ;;
-    ((and (smart-lisp-mode-p) (smart-lisp-at-tag-p)) .
-     ((smart-lisp) . (smart-lisp 'show-doc)))
+    ((or (and (smart-lisp-mode-p) (smart-lisp-at-tag-p))
+	 ;; Tightly limit Lisp matches in change-log-mode.
+	 (smart-lisp-at-change-log-tag-p)) .
+	 ((smart-lisp) . (smart-lisp 'show-doc)))
+    ;;
     ;;
     ((and (eq major-mode 'java-mode) buffer-file-name
 	  (or (smart-java-at-tag-p)
@@ -275,11 +292,6 @@ Its default value is #'smart-scroll-down."
 	  (smart-javascript-at-tag-p)) .
 	  ((smart-javascript) . (smart-javascript nil 'next-tag)))
     ;;
-    ((and (or (and (eq major-mode 'python-mode) buffer-file-name)
-	      (string-match "^Pydoc:\\|\\*?Python" (buffer-name)))
-	  (smart-python-at-tag-p)) .
-	  ((smart-python) . (smart-python nil 'next-tag)))
-    ;;
     ((and (eq major-mode 'objc-mode) buffer-file-name
 	  (smart-objc-at-tag-p)) .
 	  ((smart-objc) . (smart-objc nil 'next-tag)))
@@ -384,13 +396,14 @@ evaluated.
 The `hkey-alist' variable is the subset of this alist used by the
 smart keyboard keys.")
 
-;; This must be required after hmouse-alist is defined since this will
-;; recursively require hmouse-drv which requires hui-window when being
-;; compiled and that library requires that hmouse-alist be defined.
+;; This must be required after hmouse-alist is defined in this file
+;; since this will recursively require hmouse-drv which requires
+;; hui-window when being compiled and that library requires that
+;; hmouse-alist be defined.
 (require 'hmouse-key)
 
 ;; This next library adds drag actions to `hmouse-alist'.
-(load "hui-window")
+(require 'hui-window)
 
 ;;; ************************************************************************
 ;;; support code
@@ -590,7 +603,7 @@ If key is pressed:
 	((< (current-column) 5) (calendar-cursor-to-nearest-date)
 	 (scroll-calendar-right-three-months 1))
 	(t (calendar-cursor-to-nearest-date)
-	   (view-diary-entries 1))))
+	   (diary-view-entries 1))))
 
 (defun smart-calendar-assist ()
   "Uses a single assist-key or mouse assist-key to manipulate the scrolling calendar.
@@ -613,6 +626,30 @@ If assist-key is pressed:
 	 (scroll-calendar-left-three-months 1))
 	(t (mark-diary-entries))))
 
+;;; ************************************************************************
+;;; smart-company mode functions
+;;; ************************************************************************
+
+;; These functions are called from hkey-alist when keyboard Smart Keys
+;; are used.  For mouse keys, they are bound to local keys in
+;; company-mode's minor mode map.
+
+(defun smart-company-to-definition (event)
+  "Action Key binding for company-mode completions popup to show item definition.
+Use left mouse key, RET or TAB key to select a completion and exit."
+  (interactive "e")
+  (when (mouse-event-p last-command-event)
+    (company-select-mouse event))
+  (company-show-location))
+
+(defun smart-company-help (event)
+  "Assist Key binding for company-mode completions popup to show item doc."
+  (interactive "e")
+  (when (mouse-event-p last-command-event)
+    (company-select-mouse event))
+  (if (featurep 'company-quickhelp)
+      (company-quickhelp-manual-begin)
+    (company-show-doc-buffer)))
 
 ;;; ************************************************************************
 ;;; smart-dired functions
@@ -702,9 +739,13 @@ If assist-key is pressed:
 	 (goto-char (point-max)))
 	((looking-at "~") (dired-flag-backup-files))
 	((looking-at "#") (dired-flag-auto-save-files))
-	(t (if (fboundp 'dired-flag-file-deletion)
-	       (dired-flag-file-deletion 1)
-	     (dired-flag-file-deleted 1)))))
+	(t 
+	 ;; Prevent any region selection from causing multiple files
+	 ;; to be marked for deletion; we want to mark only one.
+	 (deactivate-mark t)
+	 (if (fboundp 'dired-flag-file-deletion)
+	     (dired-flag-file-deletion 1)
+	   (dired-flag-file-deleted 1)))))
 
 ;;; ************************************************************************
 ;;; smart-gnus functions
